'use strict';var url=require('url'),server=require('@cdm-logger/server'),ioredisEngine=require('../engines/ioredis-engine.cjs'),envConfig=require('../config/env-config.cjs');function _interopNamespaceDefault(e){var n=Object.create(null);if(e){Object.keys(e).forEach(function(k){if(k!=='default'){var d=Object.getOwnPropertyDescriptor(e,k);Object.defineProperty(n,k,d.get?d:{enumerable:true,get:function(){return e[k]}});}})}n.default=e;return Object.freeze(n)}var url__namespace=/*#__PURE__*/_interopNamespaceDefault(url);class Cache {
    static DEFAULT_SCOPE = 'cde_cache';
    static instance;
    logger;
    engine;
    constructor(engine, logger) {
        this.engine = engine;
        if (logger) {
            this.logger = logger.child(this.constructor.name);
        }
        else {
            this.logger = server.logger.child(this.constructor.name);
        }
    }
    static get Instance() {
        if (this.instance) {
            return this.instance;
        }
        const { port, hostname } = url__namespace.parse(envConfig.config.REDIS_URL);
        let options;
        let client;
        if (envConfig.config.REDIS_SENTINEL_ENABLED) {
            options = {
                sentinels: [{ host: hostname, port }],
                name: envConfig.config.REDIS_SENTINEL_NAME,
            };
            client = new ioredisEngine.Redis(options);
        }
        else if (envConfig.config.REDIS_CLUSTER_ENABLED) {
            options = {};
            client = new ioredisEngine.Redis(options, true, envConfig.config.REDIS_CLUSTER_URL);
        }
        else {
            options = { host: hostname, port };
            client = new ioredisEngine.Redis(options);
        }
        this.instance = new this(client);
        return this.instance;
    }
    key(key, scope) {
        return `${scope || Cache.DEFAULT_SCOPE}.${key}`;
    }
    isExpired(createdAt, maxAge) {
        const now = parseInt(`${Date.now() / 1000}`, null);
        return now - createdAt > maxAge;
    }
    getOptions(opts) {
        return Object.assign(this.defaults, opts || {});
    }
    get defaults() {
        return {
            scope: Cache.DEFAULT_SCOPE,
            maxAge: envConfig.config.REDIS_CACHE_MAX_AGE_in_sec,
            createdAt: parseInt(`${Date.now() / 1000}`, null),
        };
    }
    async set(key, payload, options) {
        const opts = this.getOptions(options);
        const cache = Object.assign({}, { payload }, opts);
        const path = this.key(key, opts.scope);
        this.engine.set(path, cache);
        return payload;
    }
    async get(key, options) {
        const opts = this.getOptions(options);
        const path = this.key(key, opts.scope);
        const cache = await this.engine.get(path);
        if (cache == null) {
            return null;
        }
        if (this.isExpired(cache.createdAt, cache.maxAge)) {
            this.engine.drop(path);
            return null;
        }
        else {
            return opts.parse
                ? cache.payload
                : cache.payload; // not really doing anything here
        }
    }
    async drop(key, options) {
        const opts = this.getOptions(options);
        const path = this.key(key, opts.scope);
        this.engine.drop(path);
        return true;
    }
    async invalidate(keys, options) {
        if (keys.length <= 0) {
            return this.engine.invalidate();
        }
        else {
            const opts = this.getOptions(options);
            const paths = keys.map(key => this.key(key, opts.scope));
            return this.engine.invalidate(paths);
        }
    }
}exports.Cache=Cache;//# sourceMappingURL=cache-service.cjs.map
